// -*- C++ -*-
//
//  This file is part of Siena, a wide-area event notification system.
//  See http://www.inf.usi.ch/carzaniga/siena/
//
//  Authors: Antonio Carzaniga
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2001-2003 University of Colorado
//
//  Siena is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//  
//  Siena is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with Siena.  If not, see <http://www.gnu.org/licenses/>.
//
#ifndef SIENA_BCTABLE_H
#define SIENA_BCTABLE_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#else 
#define CONFIG_BLOOM_FILTER_SIZE 256
#endif

#include <siena/forwarding.h>
#include <siena/allocator.h>
#include <siena/btable.h>

namespace siena {

class BCFilterList;

/** @brief implementation of the forwarding table based on Bloom
 *         filters and counting algorithm.
 *
 *  Like BTable, this implementation is based on encoded filters and
 *  messages.  However, instead of using a linear scan of the set of
 *  encoded filters, this implementation uses a data structure and
 *  matching algorithm that are vary similar to those implemented in
 *  FwdTable.
 *
 *  \b WARNING: This implementation is provided primarily for
 *  experimental purposes.  See BTable for details.
 **/
class BCTable : public BTable {
public:
    BCTable();
    virtual ~BCTable();

    virtual void consolidate();
    virtual void match(const message &, MatchHandler &) const;

private:
    /** @brief list of predicates.
     *
     *  each link in this linked list is a pair <predicate,interface-id>
     **/
    BCFilterList *	positions[CONFIG_BLOOM_FILTER_SIZE];
    unsigned int	f_counter;
    unsigned int	if_counter;
};

} // end namespace siena

#endif
